package internal

import (
	"context"
	"encoding/json"
	"fmt"
	"os"
	"time"

	"github.com/minio/minio-go/v7"
	"github.com/nats-io/nats.go"
)

func Process(ctx context.Context, mc *minio.Client, nc *nats.Conn, dryRun bool) {
	_, err := nc.Subscribe("minioevents", func(m *nats.Msg) {
		// fmt.Printf("Received a message: %s\n", string(m.Data))
		var event Event
		if err := json.Unmarshal(m.Data, &event); err != nil {
			fmt.Println("wrong event format")
			return
		}
		bucket, objectKey := event.Bucket(), event.ObjectKey()
		if bucket == "" || objectKey == "" {
			return
		}

		fmt.Printf("[%s/%s] cleanning triggered by %s\n", bucket, objectKey, event.Name)

		now := time.Now()
		versions := mc.ListObjects(ctx, bucket, minio.ListObjectsOptions{
			Prefix:       objectKey,
			WithVersions: true,
			WithMetadata: true,
		})

		var lastKept *minio.ObjectInfo
		var total, removed int
		for version := range versions {
			if version.Key != objectKey {
				continue
			}
			total++
			v := version
			if v.IsLatest || lastKept == nil {
				lastKept = &v
				continue
			}
			removeVersion := func(reason string) {
				if !dryRun {
					err := mc.RemoveObject(ctx, bucket, version.Key, minio.RemoveObjectOptions{
						VersionID: version.VersionID,
					})
					if err != nil {
						fmt.Println(err)
						return
					}
				}
				removed++
				fmt.Printf("[%s/%s] removed version %s from %s because is %s (%d/%d)\n",
					bucket, version.Key, version.VersionID, version.LastModified, reason, removed, total)
			}
			nowDiff := version.LastModified.Sub(now).Abs()
			_, w1 := lastKept.LastModified.ISOWeek()
			_, w2 := version.LastModified.ISOWeek()
			switch {
			// remove duplicates
			case lastKept.ETag == v.ETag:
				removeVersion("same etag")
			// between 5 & 30 days keep a version per day
			case nowDiff.Hours() > 120 && nowDiff.Hours() < 720 && lastKept.LastModified.Year() == version.LastModified.Year() && lastKept.LastModified.YearDay() == version.LastModified.YearDay():
				removeVersion("same day in interval 5 to 30 days")
			// after 30 days keep a version per week
			case nowDiff.Hours() > 720 && lastKept.LastModified.Year() == version.LastModified.Year() && w1 == w2:
				removeVersion("same week and > 30 days")
			default:
				lastKept = &v
			}

		}
		fmt.Printf("[%s/%s] finished in %v\n", bucket, objectKey, time.Since(now))
	})
	if err != nil {
		fmt.Println(err)
		os.Exit(1)
	}

	<-ctx.Done()

	fmt.Println("terminated")
}
